      !------------------------- INT_FF_EXEC --------------------------

      subroutine int_ff_exec(fname)

      !--- Declarations
      
      character*(*) fname

      include 'stack.h'

      integer topk,rhsk,topl,l1,nlr1
      logical checkrhs,checklhs
      logical getsmat

      !--- Corps de 'int_ff_exec'

      rhs = max(0,rhs)

      topk = top 
      rhsk = rhs

      nbvars = 0
      minrhs = 1
      maxrhs = 1
      maxlhs = 1

      if (.not. checkrhs(fname,minrhs,maxrhs)) return
      if (.not. checklhs(fname,1,maxlhs)) return

      if (.not. getsmat(fname,topk,top-rhs+1,m1,n1,1,1,l1,nlr1))
     $   return

      buf = ' '
      il1 = istk(l1-1)
      call in2str(il1-1,istk(l1),buf)
      
      call freefem_code(buf,err)

      if(err .gt. 0) then 
         call error(999)
         return
      endif

      topk = top - rhs
      topl = top + 0
      !Putting in order the stack
      top = topk + lhs

      end

      !------------------------ INT_FF_PROBLEM ------------------------

      subroutine int_ff_problem(fname)

      !--- Declarations

      character*(*) fname

      include 'stack.h'
      
      integer topk,rhsk,topl,l1,nlr1
      logical checkrhs,checklhs
      logical getsmat,getrhsvar
      integer default

      !--- Corps de 'int_ff_problem'

      rhs = max(0,rhs)

      topk = top 
      rhsk = rhs

      nbvars = 0
      minrhs = 1
      maxrhs = 2
      maxlhs = 1

      if (.not. checkrhs(fname,minrhs,maxrhs)) return
      if (.not. checklhs(fname,1,maxlhs)) return
      
      if (.not. getsmat(fname,topk,top-rhs+1,m1,n1,1,1,l1,nlr1))
     $   return

      buf = ' '
      il1 = istk(l1-1)
      call in2str(il1-1,istk(l1),buf)
     
      if (rhs .eq. 1) then
         default=0
         call ff_problem(buf,default,err)
      else 
         if (rhs .eq. 2) then
            if (.not. getrhsvar(2,'b',m,n,lchoice)) return
            call ff_problem(buf,istk(lchoice),err)
         else
            buf = fname // 'two many arguments'
            err = 1
         end if
      end if

      if(err .gt. 0) then 
         call error(999)
         return
      endif

      topk = top - rhs
      topl = top + 0
      !Putting in order the stack
      top = topk + lhs

      end

      !-------------------------- INT_FF_END --------------------------

      subroutine int_ff_end(fname)

      !--- Declarations

      character*(*) fname

      include 'stack.h'
      
      logical checkrhs,checklhs
      logical putlhsvar,createvar

      !--- Corps de 'int_ff_end'

      nbvars = 0
      minrhs = 0
      maxrhs = 0
      maxlhs = 1

      if (.not. checkrhs(fname,minrhs,maxrhs)) return
      if (.not. checklhs(fname,1,maxlhs)) return

      if (.not. createvar(1,'i',m1,n1,l1)) return

      call delete_scilabana(istk(l1))

      if(istk(l1) .gt. 0) then 
         buf = 'Analyseur syntaxique FreeFem deja detruit ...\r\n'
         call error(999)
         return
      endif

      lhsvar(1) = 1
      if (.not. putlhsvar()) return
      return

      end

      !---------------------- INT_GET_FF_RESULT -----------------------

      subroutine int_get_ff_result(fname)

      !--- Declarations

      character*(*) fname

      include 'stack.h'
      
      logical checkrhs,checklhs
      logical putlhsvar,createvarfromptr,getrhsvar
      double precision lNode,lTriangle,lFunc
      character*1 str

      !--- Corps de 'int_get_ff_result'

      nbvars = 0
      minrhs = 0
      maxrhs = 1
      minlhs = 1
      maxlhs = 3

      if (.not. checkrhs(fname,minrhs,maxrhs)) return
      if (.not. checklhs(fname,minlhs,maxlhs)) return

      if (lhs .eq. 2) then
         if (rhs .eq. 1) then
            buf = ' '
            buf = fname//' : No input parameter needed in this'//
     $            ' configuration'
            call error(999)
            return
         end if
      else
         if (rhs .eq. 0) then
            buf = ' '
            buf = fname//' : You need to specify on which data'//
     $            ' we get the result'
            call error(999)
            return
         end if
         if (.not. getrhsvar(1,'c',m0,n0,l0)) return
      end if
      
      buf = ' '
      if (lhs .ne. 2) then
         call get_ff_result(cstk(l0:l0+m0*n0),lhs,
     $                      lNode,mNode,nNode,
     $                      lFunc,mFunc,nFunc,
     $                      lTriangle,mTriangle,nTriangle,
     $                      buf,err)
      else
         str='0'
         call get_ff_result(str,lhs,
     $                      lNode,mNode,nNode,
     $                      lFunc,mFunc,nFunc,
     $                      lTriangle,mTriangle,nTriangle,
     $                      buf,err)
      end if

      if(err .gt. 0) then 
         call error(999)
         return
      endif

      if (lhs .ne. 1) then
        if (.not. createvarfromptr(maxrhs+1,'d',mNode,nNode,lNode)) 
     $     return
        call freeptr(lNode)
        if (.not. createvarfromptr(maxrhs+2,'i',mTriangle,nTriangle,
     $                            lTriangle)) return
        call freeptr(lTriangle)
      end if

      if (lhs .ne. 2) then
        if (.not. createvarfromptr(maxrhs+lhs,'d',mFunc,nFunc,lFunc)) 
     $     return
        call freeptr(lFunc)
      end if
      
      if (lhs .eq. 1) then
         lhsvar(1) = maxrhs+1
      else if (lhs .eq. 2) then
         lhsvar(1) = maxrhs+1
         lhsvar(2) = maxrhs+2
      else if (lhs .eq. 3) then
         lhsvar(1) = maxrhs+1
         lhsvar(2) = maxrhs+2
         lhsvar(3) = maxrhs+3
      end if

      if (.not. putlhsvar()) return
      end

      !------------------------- INT_PUT_MESH --------------------------

      subroutine int_put_mesh(fname)

      !--- Declarations
      
      character*(*) fname

      include 'stack.h'

      integer iadr
      integer topk,rhsk,topl
      logical checkrhs,checklhs,getrhsvar
      logical getilist,getlistmat,getlistsimat,getlistscalar
      character bordername *20,fct *80

      !--- Corps de 'int_put_mesh'

      iadr(l)=l+l-1
      rhs = max(0,rhs)

      topk = top 
      rhsk = rhs
      
      if (.not. checkrhs(fname,2,2)) return
      if (.not. checklhs(fname,1,1)) return
      if (.not. getilist(fname,topk,top-rhs+1,n1,1,il1)) return
      
      call delete_lb_lobj
      call delete_lexp

      do i = 1,n1 - 1
         if (.not. getlistsimat(fname,topk,top-rhs+1,1,m1e1,n1e1,i+1,1,
     $                          lr1e1,me1)) return
         call in2str(me1,istk(lr1e1),bordername)

         if (.not. getilist(fname,topk,top-rhs+1,n1,i+1,il)) return
                  
         lk = lstk(top)
         lstk(top) = il
         
         if (.not. getilist(fname,topk,top,n1,1,ili)) return
         il1 = iadr(ili)
         if (istk(il1) .lt. 0) il1 = iadr(istk(il1+1))

         if (istk(il1) .eq. 1) then
            if (.not. getlistmat(fname,topk,top,1,it1e2,m1e2,n1e2,
     $                  lr1e2,lc1e2)) return
            ! Cas ou l'on donne le tableau de coordonnees
            if (.not. getlistscalar(fname,topk,top,2,lr1e3)) return
            call entier(1,stk(lr1e3),istk(iadr(lr1e3)))

            buf = ' '
            call put_scilab_border_1(bordername(1:me1),stk(lr1e2),
     $                               m1e2,istk(iadr(lr1e3)),buf,err)

            if (err .gt. 0) then 
               call error(999)
               return
            endif
         else if (istk(il1) .eq. 10) then
            ! Cas ou l'on donne la syntaxe freefem
            if (.not. getlistsimat(fname,topk,top,1,m1e4,n1e4,
     $                             1,1,lr1e4,me4)) return
            call in2str(me4,istk(lr1e4),fct)

            if (.not. getlistscalar(fname,topk,top,2,lr1e5)) return

            if (.not. getlistscalar(fname,topk,top,3,lr1e6)) return

            if (.not. getlistscalar(fname,topk,top,4,lr1e7)) return
            call entier(1,stk(lr1e7),istk(iadr(lr1e7)))

            if (.not. getlistscalar(fname,topk,top,5,lr1e8)) return 
            call entier(1,stk(lr1e8),istk(iadr(lr1e8)))

            buf = ' '
            call put_scilab_border_2(bordername(1:me1),
     $                               fct(1:me4),
     $                               stk(lr1e5),stk(lr1e6),
     $                               istk(iadr(lr1e7)),
     $                               istk(iadr(lr1e8)),
     $                               buf,err)
         else
            return
         end if

         lstk(top) = lk
      end do

      buf = ' '
      if (.not. getrhsvar(2,'c',m1,n1,l1)) return
      call build_scilab_mesh(cstk(l1:l1+m1*n1),buf,err)

      if (err .gt. 0) then 
         call error(999)
         return
      endif

      topk = top - rhs
      topl = top + 0
      !Putting in order the stack
      top = topk + lhs

      return
      end

      !----------------------- INT_PUT_NEW_MESH ------------------------

      subroutine int_put_new_mesh(fname)

      !--- Declarations
      
      character*(*) fname

      include 'stack.h'

      integer iadr
      integer topk,rhsk,topl
      logical checkrhs,checklhs,getrhsvar
      logical getilist,getlistmat,getlistsimat,getlistscalar
      character bordername *20

      !--- Corps de 'int_put_new_mesh'

      iadr(l)=l+l-1
      rhs = max(0,rhs)

      topk = top 
      rhsk = rhs
      
      if (.not. checkrhs(fname,2,2)) return
      if (.not. checklhs(fname,1,1)) return
      if (.not. getilist(fname,topk,top-rhs+1,n1,1,il1)) return
      
      call delete_lb_lobj

      do i = 1,n1 - 1
         if (.not. getlistsimat(fname,topk,top-rhs+1,1,m1e1,n1e1,i+1,1,
     $                          lr1e1,me1)) return
         call in2str(me1,istk(lr1e1),bordername)

         if (.not. getilist(fname,topk,top-rhs+1,n1,i+1,il)) return
                  
         lk = lstk(top)
         lstk(top) = il
         
         if (.not. getilist(fname,topk,top,n1,1,ili)) return
         il1 = iadr(ili)
         if (istk(il1) .lt. 0) il1 = iadr(istk(il1+1))

         if (istk(il1) .eq. 1) then
            if (.not. getlistmat(fname,topk,top,1,it1e2,m1e2,n1e2,
     $                  lr1e2,lc1e2)) return
            ! Cas ou l'on donne le tableau de coordonnees
            if (.not. getlistscalar(fname,topk,top,2,lr1e3)) return
            call entier(1,stk(lr1e3),istk(iadr(lr1e3)))

            buf = ' '
            call put_scilab_border_1(bordername(1:me1),stk(lr1e2),
     $                               m1e2,istk(iadr(lr1e3)),buf,err)

            if (err .gt. 0) then 
               call error(999)
               return
            endif
         else if (istk(il1) .eq. 10) then
            ! Cas ou l'on donne la syntaxe freefem
            call add_lobj
         else
            return
         end if

         lstk(top) = lk
      end do

      buf = ' '
      if (.not. getrhsvar(2,'c',m1,n1,l1)) return
      call build_scilab_mesh(cstk(l1:l1+m1*n1),buf,err)

      if (err .gt. 0) then 
         call error(999)
         return
      endif

      topk = top - rhs
      topl = top + 0
      !Putting in order the stack
      top = topk + lhs

      return
      end

      !------------------------ INT_GET_MATRIX -------------------------

      subroutine int_get_matrix(fname)

      !--- Declarations

      character*(*) fname

      include 'stack.h'
      
      logical checkrhs,checklhs
      logical putlhsvar,createvarfromptr
      double precision lMatrix,ljlow,ljhigh,lblock

      !--- Corps de 'int_get_matrix'

      nbvars = 0
      minrhs = 0
      maxrhs = 0
      minlhs = 4
      maxlhs = 4

      if (.not. checkrhs(fname,minrhs,maxrhs)) return
      if (.not. checklhs(fname,minlhs,maxlhs)) return

      buf=' '
      call get_matrix(lMatrix,mMatrix,
     $                ljlow,ljhigh,lsize,
     $                lblock,buf,err)

      if(err .gt. 0) then 
         call error(999)
         return
      endif

      if (.not. createvarfromptr(1,'d',mMatrix,1,lMatrix)) return
      call freeptr(lMatrix)
      if (.not. createvarfromptr(2,'i',lsize,1,ljlow)) return
      call freeptr(ljlow)
      if (.not. createvarfromptr(3,'i',lsize,1,ljhigh)) return
      call freeptr(ljhigh)
      if (.not. createvarfromptr(4,'i',1,1,lblock)) return
      call freeptr(lblock)
      
      lhsvar(1) = 1
      lhsvar(2) = 2
      lhsvar(3) = 3
      lhsvar(4) = 4
      if (.not. putlhsvar()) return

      end


                            !********************!      
                            ! INTERFACE FUNCTION !
                            !********************!
      subroutine freefemfi

      !--- Declarations
      
      include 'stack.h'

      !--- Corps de 'femfi'
      
      rhs = max(0,rhs)

      goto (1,2,3,4,5,6,7) fin
      return
 1    call int_ff_exec('ff_exec')
      return
 2    call int_ff_problem('ff_problem')
      return
 3    call int_ff_end('ff_end')
      return
 4    call int_get_ff_result('getffResult')
      return
 5    call int_put_mesh('buildMesh')
      return
 6    call int_put_new_mesh('UpdateMesh')
      return
 7    call int_get_matrix('getMatrix')
      return

      end
