#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <strings.h>
#include "def.h"

void * load_lib(int argc, char * argv [])
{
    void * lib_handle;
    char lib_path [MAXPATHLEN];
    char * env_ptr;

    if (argc > 1)
	strcpy(lib_path,argv[1]);
    else
    {
	env_ptr = getenv("SHLPATH");
	if (env_ptr != NULL)
	    strcpy(lib_path,env_ptr);
	else
	{
	    printf("Shared library to use >> ");
	    scanf("%s",lib_path);
	}
    }

    lib_handle = dlopen(lib_path,RTLD_NOW);
    if ( lib_handle == (void *) NULL || lib_handle < (void *) 0 )
    {
	perror("sh_load : error loading library -> ");
	switch(errno)
 	{
 	    case ENOEXEC : perror("ENOEXEC\n"); break;
 	    case ENOSYM : perror("ENOSYM\n"); break;
 	    case ENOMEM : perror("ENOMEM\n"); break;
 	    case EINVAL : perror("EINVAL\n"); break;
 	    case ENOENT : printf("ENOENT\n"); break;
 	    case EACCES : printf("EACCESS\n"); break;
 	    default : printf("Unknown\n");
        }
	exit(1);
    }
    printf("shared library %s loaded\n", lib_path);
	
    return lib_handle;
}

void * mymalloc()
{
    return malloc(10000);
}

int show_symbols(shl_t hndl, short type, int flags)
{
    int num_symbols, sym_idx;
    struct shl_symbol * symbols;

    num_symbols = shl_getsymbols(hndl, type, flags,mymalloc, &symbols);
    if (num_symbols < 0)
    {
	printf("shl_getsymbols failed\n");
	exit(1);
    }


    for (sym_idx = 0; sym_idx < num_symbols; sym_idx++)
    {
	printf("     %30s",symbols -> name);
	if (type == TYPE_UNDEFINED)
          switch(symbols -> type)
	  {
	    case TYPE_PROCEDURE:
		printf("  PROCEDURE");
		break;
	    case TYPE_DATA:
		printf("  DATA     ");
		break;
	    case TYPE_STORAGE:
		printf("  STORAGE  ");
		break;
	  }

        if ((flags & EXPORT_SYMBOLS) && (flags & NO_VALUES) == 0)
	    printf("  0x%8x",symbols -> value);
        
	printf("\n");
	symbols++;
    }

    free(symbols);

    return num_symbols;
}

int main(int argc, char ** argv)
{
    void (*load_data)();

  shl_t handle = load_lib(argc,argv);
  shl_findsym(&handle,"__do_global_ctors",TYPE_PROCEDURE, (void *) &load_data);
  if (load_data) (*load_data)();

  printf("  Imports : \n");
  int num_syms = show_symbols(handle, TYPE_UNDEFINED, 
			          IMPORT_SYMBOLS);
  printf("     TOTAL SYMBOLS : %d\n", num_syms);
  printf("   Exports :\n");
  num_syms = show_symbols(handle, TYPE_UNDEFINED,
        		  EXPORT_SYMBOLS);
  printf("     TOTAL SYMBOLS : %d\n", num_syms);
 
  int (*main1) ();
  if (shl_findsym(&handle,"main1", TYPE_PROCEDURE,(void *) &main1))
      perror("shl_load : error finding main1") ,exit(1);

  printf("\n\n--------- Donnees Counter\n");
  int * Co;
  if (shl_findsym(&handle,"Counter", TYPE_DATA,(void *) &Co))
      perror("shl_load : error finding Counter") ,exit(1);
  printf("Initial Counter : %d\n",*Co);
  (*Co)++;
  printf("New Counter : %d\n",*Co);

  printf("--------- Donnees essai\n");
  A * c;
  if (shl_findsym(&handle,"essai", TYPE_DATA,(void *) &c))
      perror("shl_load : error finding essai") ,exit(1);
  cout << "Adresse de essai (ex2.c) : " << c << endl;
  c -> state();
  c -> Count = 4;
  c -> state();

  printf("--------- Execution du main\n");
  printf("valeur de retour du main : %d \n", (*main1)() );

  shl_findsym(&handle,"__do_global_dtors",TYPE_PROCEDURE, (void *) &load_data);
  if (load_data) (*load_data)();
  shl_unload(handle);
  return 0;
}
